@@ -293,6 +293,7 @@ CodeDataLoggerDialog_t::~CodeDataLoggerDialog_t(void)
 	updateTimer->stop();
 
 	//printf("Code Data Logger Window Deleted\n");
+	cdlWin = NULL;
 }
 //----------------------------------------------------
 void CodeDataLoggerDialog_t::closeEvent(QCloseEvent *event)
@@ -303,6 +304,7 @@ void CodeDataLoggerDialog_t::closeEvent(QCloseEvent *event)
 	done(0);
 	deleteLater();
 	event->accept();
+	cdlWin = NULL;
 }
 //----------------------------------------------------
 void CodeDataLoggerDialog_t::closeWindow(void)
@@ -312,6 +314,7 @@ void CodeDataLoggerDialog_t::closeWindow(void)
 	settings.setValue("cdLogger/geometry", saveGeometry());
 	done(0);
 	deleteLater();
+	cdlWin = NULL;
 }
 //----------------------------------------------------
 void CodeDataLoggerDialog_t::autoSaveCdlStateChange(int state)
@@ -476,10 +479,10 @@ void CodeDataLoggerDialog_t::saveCdlFileAs(void)
 	}
 	//qDebug() << "selected file path : " << filename.toUtf8();
 
-	fceuWrapperLock();
+	FCEU_WRAPPER_LOCK();
 	strcpy(loadedcdfile, filename.toStdString().c_str());
 	SaveCDLogFile();
-	fceuWrapperUnLock();
+	FCEU_WRAPPER_UNLOCK();
 }
 //----------------------------------------------------
 void CodeDataLoggerDialog_t::loadCdlFile(void)
@@ -531,9 +534,9 @@ void CodeDataLoggerDialog_t::loadCdlFile(void)
 	}
 	//qDebug() << "selected file path : " << filename.toUtf8();
 
-	fceuWrapperLock();
+	FCEU_WRAPPER_LOCK();
 	LoadCDLog(filename.toStdString().c_str());
-	fceuWrapperUnLock();
+	FCEU_WRAPPER_UNLOCK();
 
 	return;
 }
@@ -754,7 +757,7 @@ static int getDefaultCDLFile(char *filepath)
 //----------------------------------------------------
 void FreeCDLog(void)
 {
-	fceuWrapperLock();
+	FCEU_WRAPPER_LOCK();
 	if (cdloggerdata)
 	{
 		free(cdloggerdata);
@@ -767,14 +770,14 @@ void FreeCDLog(void)
 		cdloggervdata = NULL;
 		cdloggerVideoDataSize = 0;
 	}
-	fceuWrapperUnLock();
+	FCEU_WRAPPER_UNLOCK();
 }
 //----------------------------------------------------
 void InitCDLog(void)
 {
 	int rom_sel = 0;
 
-	fceuWrapperLock();
+	FCEU_WRAPPER_LOCK();
 	if (GameInfo->type == GIT_FDS)
 	{
 		rom_sel = 1;
@@ -794,7 +797,7 @@ void InitCDLog(void)
 			cdloggervdata = (unsigned char *)malloc(8192);
 		}
 	}
-	fceuWrapperUnLock();
+	FCEU_WRAPPER_UNLOCK();
 }
 //----------------------------------------------------
 void ResetCDLog(void)
@@ -803,7 +806,7 @@ void ResetCDLog(void)
 	{
 		return;
 	}
-	fceuWrapperLock();
+	FCEU_WRAPPER_LOCK();
 
 	codecount = datacount = rendercount = vromreadcount = 0;
 	undefinedcount = cdloggerdataSize;
@@ -829,7 +832,7 @@ void ResetCDLog(void)
 			memset(cdloggervdata, 0, 8192);
 		}
 	}
-	fceuWrapperUnLock();
+	FCEU_WRAPPER_UNLOCK();
 }
 //----------------------------------------------------
 bool LoadCDLog(const char *nameo)
@@ -882,12 +885,12 @@ bool LoadCDLog(const char *nameo)
 //----------------------------------------------------
 void StartCDLogging(void)
 {
-	fceuWrapperLock();
+	FCEU_WRAPPER_LOCK();
 	FCEUI_SetLoggingCD(1);
 	//EnableTracerMenuItems();
 	//SetDlgItemText(hCDLogger, BTN_CDLOGGER_START_PAUSE, "Pause");
 	autoSaveArmedCDL = true;
-	fceuWrapperUnLock();
+	FCEU_WRAPPER_UNLOCK();
 }
 //----------------------------------------------------
 bool PauseCDLogging(void)
@@ -898,11 +901,11 @@ bool PauseCDLogging(void)
 	//	MessageBox(hCDLogger, "The Trace Logger is currently using this for some of its features.\nPlease turn the Trace Logger off and try again.","Unable to Pause Code/Data Logger", MB_OK);
 	//	return false;
 	//}
-	fceuWrapperLock();
+	FCEU_WRAPPER_LOCK();
 	FCEUI_SetLoggingCD(0);
 	//EnableTracerMenuItems();
 	//SetDlgItemText(hCDLogger, BTN_CDLOGGER_START_PAUSE, "Start");
-	fceuWrapperUnLock();
+	FCEU_WRAPPER_UNLOCK();
 	return true;
 }
 //----------------------------------------------------
