@@ -39,6 +39,7 @@
 #include <QFileDialog>
 #include <QInputDialog>
 #include <QMessageBox>
+#include <QShortcut>
 #include <QPainter>
 #include <QGuiApplication>
 
@@ -152,6 +153,7 @@ TraceLoggerDialog_t::TraceLoggerDialog_t(QWidget *parent)
 	QAction *act;
 	QLabel *lbl;
 	int opt, useNativeMenuBar;
+	QShortcut *shortcut;
 
 	if (recBufMax == 0)
 	{
@@ -198,6 +200,12 @@ TraceLoggerDialog_t::TraceLoggerDialog_t(QWidget *parent)
 
 	connect(hbar, SIGNAL(valueChanged(int)), this, SLOT(hbarChanged(int)));
 	connect(vbar, SIGNAL(valueChanged(int)), this, SLOT(vbarChanged(int)));
+	connect(vbar, SIGNAL(actionTriggered(int)), this, SLOT(vbarActionTriggered(int)));
+
+	shortcut = new QShortcut( QKeySequence("Page Up"), this );
+	connect( shortcut, SIGNAL(activated(void)), this, SLOT(pageUpActivated(void)) );
+	shortcut = new QShortcut( QKeySequence("Page Down"), this );
+	connect( shortcut, SIGNAL(activated(void)), this, SLOT(pageDnActivated(void)) );
 
 	traceView->setScrollBars(hbar, vbar);
 	traceView->setMinimumHeight(256);
@@ -398,7 +406,7 @@ TraceLoggerDialog_t::~TraceLoggerDialog_t(void)
 
 	traceLogWindow = NULL;
 
-	printf("Trace Logger Window Deleted\n");
+	//printf("Trace Logger Window Deleted\n");
 }
 //----------------------------------------------------
 void TraceLoggerDialog_t::closeEvent(QCloseEvent *event)
@@ -550,7 +558,7 @@ void TraceLoggerDialog_t::openLogFile(void)
 	QString filename;
 	QFileDialog dialog(this, tr("Select Log File"));
 
-	printf("Log File Select\n");
+	//printf("Log File Select\n");
 
 	dialog.setFileMode(QFileDialog::AnyFile);
 
@@ -619,6 +627,81 @@ void TraceLoggerDialog_t::vbarChanged(int val)
 	traceView->update();
 }
 //----------------------------------------------------
+void TraceLoggerDialog_t::vbarActionTriggered(int act)
+{
+	int val = vbar->value();
+	int max = vbar->maximum();
+
+	if ( act == QAbstractSlider::SliderSingleStepAdd )
+	{
+		val = val - vbar->singleStep();
+
+		if ( val < 0 )
+		{
+			val = 0;
+		}
+		vbar->setSliderPosition(val);
+	}
+	else if ( act == QAbstractSlider::SliderSingleStepSub )
+	{
+		val = val + vbar->singleStep();
+
+		if ( val > max )
+		{
+			val = max;
+		}
+		vbar->setSliderPosition(val);
+	}
+        else if ( act == QAbstractSlider::SliderPageStepAdd )
+        {
+               	val = val - vbar->pageStep();
+
+		if ( val < 0 )
+		{
+			val = 0;
+		}
+		vbar->setSliderPosition(val);
+        }
+        else if ( act == QAbstractSlider::SliderPageStepSub )
+        {
+                val = val + vbar->pageStep();
+
+		if ( val > max )
+		{
+			val = max;
+		}
+		vbar->setSliderPosition(val);
+        }
+	//printf("ACT:%i\n", act);
+}
+//----------------------------------------------------
+void TraceLoggerDialog_t::pageUpActivated(void)
+{
+	int val = vbar->value();
+	int max = vbar->maximum();
+
+	val = val + vbar->pageStep();
+
+	if ( val > max )
+	{
+		val = max;
+	}
+	vbar->setSliderPosition(val);
+}
+//----------------------------------------------------
+void TraceLoggerDialog_t::pageDnActivated(void)
+{
+	int val = vbar->value();
+
+	val = val - vbar->pageStep();
+
+	if ( val < 0 )
+	{
+		val = 0;
+	}
+	vbar->setSliderPosition(val);
+}
+//----------------------------------------------------
 void TraceLoggerDialog_t::logToFileStateChanged(int state)
 {
 	g_config->setOption("SDL.TraceLogSaveToFile", state != Qt::Unchecked );
@@ -1751,7 +1834,7 @@ void QTraceLogView::wheelEvent(QWheelEvent *event)
 
 	if (!numPixels.isNull())
 	{
-		wheelPixelCounter -= numPixels.y();
+		wheelPixelCounter += numPixels.y();
 		//printf("numPixels: (%i,%i) \n", numPixels.x(), numPixels.y() );
 	}
 	else if (!numDegrees.isNull())
@@ -1759,7 +1842,7 @@ void QTraceLogView::wheelEvent(QWheelEvent *event)
 		//QPoint numSteps = numDegrees / 15;
 		//printf("numSteps: (%i,%i) \n", numSteps.x(), numSteps.y() );
 		//printf("numDegrees: (%i,%i)  %i\n", numDegrees.x(), numDegrees.y(), pxLineSpacing );
-		wheelPixelCounter -= (pxLineSpacing * numDegrees.y()) / (15 * 8);
+		wheelPixelCounter += (pxLineSpacing * numDegrees.y()) / (15 * 8);
 	}
 	//printf("Wheel Event: %i\n", wheelPixelCounter);
 
@@ -2220,7 +2303,9 @@ void QTraceLogView::paintEvent(QPaintEvent *event)
 	nrow = (viewHeight / pxLineSpacing);
 
 	if (nrow < 1)
+	{
 		nrow = 1;
+	}
 
 	viewLines = nrow;
 
@@ -2237,7 +2322,7 @@ void QTraceLogView::paintEvent(QPaintEvent *event)
 	else
 	{
 		vbar->setMaximum( recBufNum - viewLines );
-		vbar->setPageStep( viewLines );
+		vbar->setPageStep( (viewLines*7)/8 );
 		vbar->show();
 	}
 
@@ -2396,7 +2481,7 @@ TraceLogDiskThread_t::TraceLogDiskThread_t( QObject *parent )
 //----------------------------------------------------
 TraceLogDiskThread_t::~TraceLogDiskThread_t(void)
 {
-	printf("Disk Thread Cleanup\n");
+	//printf("Disk Thread Cleanup\n");
 #ifdef WIN32
 	if (logFile != INVALID_HANDLE_VALUE)
 	{
@@ -2424,7 +2509,7 @@ void TraceLogDiskThread_t::run(void)
 	int i,idx=0;
 	int blockSize = 4 * 1024;
 
-	printf("Trace Log Disk Start\n");
+	//printf("Trace Log Disk Start\n");
 
 	setPriority( QThread::HighestPriority );
 
@@ -2521,7 +2606,7 @@ void TraceLogDiskThread_t::run(void)
 	}
 	#endif
 	
-	printf("Trace Log Disk Exit\n");
+	//printf("Trace Log Disk Exit\n");
 	emit finished();
 }
 //----------------------------------------------------
