@@ -25,6 +25,7 @@
 
 #include "fceu.h"
 #include "ppu.h"
+#include "video.h"
 #include "../common/cheat.h"
 
 #include "Qt/input.h"
@@ -34,6 +35,7 @@
 #include "Qt/sdl-video.h"
 #include "Qt/AviRecord.h"
 #include "Qt/unix-netplay.h"
+#include "Qt/TasEditor/taseditor_config.h"
 
 #ifdef WIN32
 #include <windows.h>
@@ -468,6 +470,7 @@ GetBaseDirectory(std::string &dir)
 Config *
 InitConfig()
 {
+	TASEDITOR_CONFIG  tasCfg;
 	std::string dir, prefix, savPath, movPath;
 	Config *config;
 
@@ -499,7 +502,7 @@ InitConfig()
 	config->addOption("frameskip", "SDL.Frameskip", 0);
 	config->addOption("intFrameRate", "SDL.IntFrameRate", 0);
 	config->addOption("clipsides", "SDL.ClipSides", 0);
-	config->addOption("nospritelim", "SDL.DisableSpriteLimit", 1);
+	config->addOption("nospritelim", "SDL.DisableSpriteLimit", 0);
 	config->addOption("swapduty", "SDL.SwapDuty", 0);
 	config->addOption("ramInit", "SDL.RamInitMethod", 0);
 
@@ -526,6 +529,7 @@ InitConfig()
 	config->addOption('f', "fullscreen", "SDL.Fullscreen", 0);
 	config->addOption("videoDriver", "SDL.VideoDriver", 0);
 	config->addOption("SDL.VideoBgColor", "#000000");
+	config->addOption("SDL.VideoVsync", 1);
 
 	// set x/y res to 0 for automatic fullscreen resolution detection (no change)
 	config->addOption('x', "xres", "SDL.XResolution", 0);
@@ -556,6 +560,7 @@ InitConfig()
 	config->addOption("SDL.ShowFrameCount", 0);
 	config->addOption("SDL.ShowLagCount", 0);
 	config->addOption("SDL.ShowRerecordCount", 0);
+	config->addOption("SDL.ShowGuiMessages", 1);
 
 	// OpenGL options
 	config->addOption("opengl", "SDL.OpenGL", 1);
@@ -690,6 +695,7 @@ InitConfig()
 	config->addOption("no-config", "SDL.NoConfig", 0);
 
 	config->addOption("autoresume", "SDL.AutoResume", 0);
+	config->addOption("SDL.FamilyKeyboardFont"  , "");
     
 	// video playback
 	config->addOption("playmov", "SDL.Movie", "");
@@ -747,9 +753,72 @@ InitConfig()
 		config->addOption( buf, "");
 	}
 
+	for (unsigned int i=0; i<10; i++)
+	{
+		char buf[128];
+		sprintf(buf, "SDL.RecentTasProject%02u", i);
+
+		config->addOption( buf, "");
+	}
+	config->addOption("SDL.TasPianoRollFont", "");
+	config->addOption("SDL.TasBookmarksFont", "");
+	config->addOption("SDL.TasBranchesFont" , "");
+	config->addOption("SDL.TasPianoRollGridColor", "#808080");
+
+	config->addOption("SDL.TasAutoSaveEnabled"                         , tasCfg.autosaveEnabled );
+	config->addOption("SDL.TasAutoSavePeriod"                          , tasCfg.autosavePeriod  );
+	config->addOption("SDL.TasAutoSaveSilent"                          , tasCfg.autosaveSilent  );
+	config->addOption("SDL.TasTooltipsEnabled"                         , tasCfg.tooltipsEnabled );
+	config->addOption("SDL.TasCurrentPattern"                          , tasCfg.currentPattern  );
+	config->addOption("SDL.TasFollowPlaybackCursor"                    , tasCfg.followPlaybackCursor  );
+	config->addOption("SDL.TasTurboSeek"                               , tasCfg.turboSeek  );
+	config->addOption("SDL.TasAutoRestoreLastPlaybackPosition"         , tasCfg.autoRestoreLastPlaybackPosition  );
+	config->addOption("SDL.TasSuperImpose"                             , tasCfg.superimpose  );
+	config->addOption("SDL.TasRecordingUsePattern"                     , tasCfg.recordingUsePattern  );
+	config->addOption("SDL.TasEnableLuaAutoFunction"                   , tasCfg.enableLuaAutoFunction  );
+	config->addOption("SDL.TasDisplayBranchesTree"                     , tasCfg.displayBranchesTree  );
+	config->addOption("SDL.TasDisplayBranchScreenshots"                , tasCfg.displayBranchScreenshots  );
+	config->addOption("SDL.TasDisplayBranchDescriptions"               , tasCfg.displayBranchDescriptions  );
+	config->addOption("SDL.TasEnableHotChanges"                        , tasCfg.enableHotChanges  );
+	config->addOption("SDL.TasFollowUndoContext"                       , tasCfg.followUndoContext  );
+	config->addOption("SDL.TasFollowMarkerNoteContext"                 , tasCfg.followMarkerNoteContext  );
+	config->addOption("SDL.TasGreenzoneCapacity"                       , tasCfg.greenzoneCapacity  );
+	config->addOption("SDL.TasMaxUndoLevels"                           , tasCfg.maxUndoLevels  );
+	config->addOption("SDL.TasEnableGreenzoning"                       , tasCfg.enableGreenzoning  );
+	config->addOption("SDL.TasAutofirePatternSkipsLag"                 , tasCfg.autofirePatternSkipsLag  );
+	config->addOption("SDL.TasAutoAdjustInputAccordingToLag"           , tasCfg.autoAdjustInputAccordingToLag  );
+	config->addOption("SDL.TasDrawInputByDragging"                     , tasCfg.drawInputByDragging  );
+	config->addOption("SDL.TasCombineConsecutiveRecordingsAndDraws"    , tasCfg.combineConsecutiveRecordingsAndDraws  );
+	config->addOption("SDL.TasUse1PKeysForAllSingleRecordings"         , tasCfg.use1PKeysForAllSingleRecordings  );
+	config->addOption("SDL.TasUseInputKeysForColumnSet"                , tasCfg.useInputKeysForColumnSet  );
+	config->addOption("SDL.TasBindMarkersToInput"                      , tasCfg.bindMarkersToInput  );
+	config->addOption("SDL.TasEmptyNewMarkerNotes"                     , tasCfg.emptyNewMarkerNotes  );
+	config->addOption("SDL.TasOldControlSchemeForBranching"            , tasCfg.oldControlSchemeForBranching  );
+	config->addOption("SDL.TasBranchesRestoreEntireMovie"              , tasCfg.branchesRestoreEntireMovie  );
+	config->addOption("SDL.TasHUDInBranchScreenshots"                  , tasCfg.HUDInBranchScreenshots  );
+	config->addOption("SDL.TasAutopauseAtTheEndOfMovie"                , tasCfg.autopauseAtTheEndOfMovie  );
+	config->addOption("SDL.TasLastExportedInputType"                   , tasCfg.lastExportedInputType  );
+	config->addOption("SDL.TasLastExportedSubtitlesStatus"             , tasCfg.lastExportedSubtitlesStatus  );
+	config->addOption("SDL.TasProjectSavingOptions_SaveInBinary"       , tasCfg.projectSavingOptions_SaveInBinary  );
+	config->addOption("SDL.TasProjectSavingOptions_SaveMarkers"        , tasCfg.projectSavingOptions_SaveMarkers  );
+	config->addOption("SDL.TasProjectSavingOptions_SaveBookmarks"      , tasCfg.projectSavingOptions_SaveBookmarks  );
+	config->addOption("SDL.TasProjectSavingOptions_SaveHistory"        , tasCfg.projectSavingOptions_SaveHistory  );
+	config->addOption("SDL.TasProjectSavingOptions_SavePianoRoll"      , tasCfg.projectSavingOptions_SavePianoRoll  );
+	config->addOption("SDL.TasProjectSavingOptions_SaveSelection"      , tasCfg.projectSavingOptions_SaveSelection  );
+	config->addOption("SDL.TasProjectSavingOptions_GreenzoneSavingMode", tasCfg.projectSavingOptions_GreenzoneSavingMode  );
+	config->addOption("SDL.TasSaveCompact_SaveInBinary"                , tasCfg.saveCompact_SaveInBinary  );
+	config->addOption("SDL.TasSaveCompact_SaveMarkers"                 , tasCfg.saveCompact_SaveMarkers  );
+	config->addOption("SDL.TasSaveCompact_SaveBookmarks"               , tasCfg.saveCompact_SaveBookmarks  );
+	config->addOption("SDL.TasSaveCompact_SaveHistory"                 , tasCfg.saveCompact_SaveHistory  );
+	config->addOption("SDL.TasSaveCompact_SavePianoRoll"               , tasCfg.saveCompact_SavePianoRoll  );
+	config->addOption("SDL.TasSaveCompact_SaveSelection"               , tasCfg.saveCompact_SaveSelection  );
+	config->addOption("SDL.TasSaveCompact_GreenzoneSavingMode"         , tasCfg.saveCompact_GreenzoneSavingMode  );
+	config->addOption("SDL.TasLastAuthorName" , "");
+
 	config->addOption("_useNativeFileDialog", "SDL.UseNativeFileDialog", false);
 	config->addOption("_useNativeMenuBar"   , "SDL.UseNativeMenuBar", false);
 	config->addOption("SDL.PauseOnMainMenuAccess", false);
+	config->addOption("SDL.AutoHideMenuFullsreen", false);
 	config->addOption("SDL.ContextMenuEnable", true);
 	config->addOption("SDL.GuiStyle", "");
 	config->addOption("SDL.QtStyleSheet", "");
@@ -1006,6 +1075,7 @@ UpdateEMUCore(Config *config)
 	config->getOption("SDL.PostRenderScanlines" , &postrenderscanlines    );
 	config->getOption("SDL.VBlankScanlines"     , &vblankscanlines        );
 	config->getOption("SDL.Skip7bitOverClocking", &skip_7bit_overclocking );
+	config->getOption("SDL.ShowGuiMessages"     , &vidGuiMsgEna           );
 
 	config->getOption("SDL.PAL", &region);
 	FCEUI_SetRegion(region);
